@@ -8,7 +8,7 @@ import pytest  # type: ignore
 
 from deepfabric.graph import (
     Graph,
-    GraphArguments,
+    GraphConfig,
     GraphModel,
     Node,
     NodeModel,
@@ -17,21 +17,23 @@ from deepfabric.graph import (
 
 
 @pytest.fixture
-def topic_graph_args():
-    """Fixture for GraphArguments."""
-    return GraphArguments(
-        root_prompt="Test root topic",
-        model_name="test-model",
-        temperature=0.7,
-        graph_degree=3,
-        graph_depth=2,
-    )
+def topic_graph_params():
+    """Fixture for Graph parameters."""
+    return {
+        "topic_prompt": "Test root topic",
+        "provider": "openai",
+        "model_name": "test-model",
+        "temperature": 0.7,
+        "degree": 3,
+        "depth": 2,
+    }
 
 
 @pytest.fixture
-def topic_graph(topic_graph_args):
+def topic_graph(topic_graph_params):
     """Fixture for Graph instance."""
-    return Graph(topic_graph_args)
+    with patch("deepfabric.graph.LLMClient"):
+        return Graph(**topic_graph_params)
 
 
 class TestValidateGraphResponse:
@@ -91,70 +93,71 @@ class TestNode:
         assert child_model.parents == [1]
 
 
-class TestGraphArguments:
-    """Tests for GraphArguments model."""
+class TestGraphConfig:
+    """Tests for GraphConfig model."""
 
     def test_valid_arguments(self):
         """Test creation with valid arguments."""
-        args = GraphArguments(
-            root_prompt="Test", model_name="gpt-4", temperature=0.5, graph_degree=2, graph_depth=3
+        config = GraphConfig(
+            topic_prompt="Test", model_name="gpt-4", temperature=0.5, degree=2, depth=3
         )
-        assert args.root_prompt == "Test"
-        assert args.model_name == "gpt-4"
-        assert args.temperature == 0.5  # noqa: PLR2004
-        assert args.graph_depth == 3  # noqa: PLR2004
+        assert config.topic_prompt == "Test"
+        assert config.model_name == "gpt-4"
+        assert config.temperature == 0.5  # noqa: PLR2004
+        assert config.depth == 3  # noqa: PLR2004
 
     def test_default_arguments(self):
         """Test default values."""
-        args = GraphArguments(
-            root_prompt="Test",
-            model_name="ollama/llama3",
+        config = GraphConfig(
+            topic_prompt="Test",
+            model_name="ollama/qwen3:8b",
             temperature=0.2,
-            graph_degree=3,
-            graph_depth=2,
+            degree=3,
+            depth=2,
         )
-        assert args.model_name == "ollama/llama3"  # From constants
-        assert args.temperature == 0.2  # From constants  # noqa: PLR2004
-        assert args.graph_degree == 3  # noqa: PLR2004
-        assert args.graph_depth == 2  # noqa: PLR2004
+        assert config.model_name == "ollama/qwen3:8b"  # From constants
+        assert config.temperature == 0.2  # From constants  # noqa: PLR2004
+        assert config.degree == 3  # noqa: PLR2004
+        assert config.depth == 2  # noqa: PLR2004
 
     def test_invalid_arguments(self):
         """Test validation errors."""
         with pytest.raises(ValueError):
-            GraphArguments(
-                root_prompt="",  # Empty root_prompt
-                model_name="ollama/llama3",
+            GraphConfig(
+                topic_prompt="",  # Empty topic_prompt
+                model_name="ollama/qwen3:8b",
                 temperature=0.2,
-                graph_degree=3,
-                graph_depth=2,
+                degree=3,
+                depth=2,
             )
 
         with pytest.raises(ValueError):
-            GraphArguments(
-                root_prompt="Test",
-                model_name="ollama/llama3",
+            GraphConfig(
+                topic_prompt="Test",
+                model_name="ollama/qwen3:8b",
                 temperature=-0.1,
-                graph_degree=3,
-                graph_depth=2,
+                degree=3,
+                depth=2,
             )  # Invalid temperature
 
         with pytest.raises(ValueError):
-            GraphArguments(
-                root_prompt="Test",
-                model_name="ollama/llama3",
+            GraphConfig(
+                topic_prompt="Test",
+                model_name="ollama/qwen3:8b",
                 temperature=-0.1,
-                graph_degree=0,
-                graph_depth=2,
+                degree=0,
+                depth=2,
             )  # Invalid degree
 
 
 class TestGraph:
     """Tests for Graph class."""
 
-    def test_initialization(self, topic_graph_args):
+    def test_initialization(self, topic_graph_params):
         """Test Graph initialization."""
-        graph = Graph(topic_graph_args)
-        assert graph.args == topic_graph_args
+        with patch("deepfabric.graph.LLMClient"):
+            graph = Graph(**topic_graph_params)
+        assert graph.config.topic_prompt == topic_graph_params["topic_prompt"]
         assert graph.root.topic == "Test root topic"
         assert graph.root.id == 0
         assert len(graph.nodes) == 1
@@ -221,10 +224,11 @@ class TestGraph:
         assert data["root_id"] == 0
         assert len(data["nodes"]) == 2  # noqa: PLR2004
 
-    def test_save_and_load(self, topic_graph_args):
+    def test_save_and_load(self, topic_graph_params):
         """Test saving and loading a graph."""
         # Create and populate a graph
-        graph = Graph(topic_graph_args)
+        with patch("deepfabric.graph.LLMClient"):
+            graph = Graph(**topic_graph_params)
         node1 = graph.add_node("Child 1")
         node2 = graph.add_node("Child 2")
         graph.add_edge(0, node1.id)
@@ -238,7 +242,8 @@ class TestGraph:
             graph.save(temp_path)
 
             # Load the graph
-            loaded_graph = Graph.from_json(temp_path, topic_graph_args)
+            with patch("deepfabric.graph.LLMClient"):
+                loaded_graph = Graph.from_json(temp_path, topic_graph_params)
 
             # Verify structure
             assert len(loaded_graph.nodes) == 3  # noqa: PLR2004
@@ -297,26 +302,19 @@ class TestGraph:
 
         assert topic_graph.has_cycle() is True
 
-    @patch("deepfabric.graph.litellm.completion")
-    def test_get_subtopics_and_connections_success(self, mock_completion, topic_graph):
+    def test_get_subtopics_and_connections_success(self, topic_graph):
         """Test successful subtopic generation."""
-        # Mock LLM response
-        mock_response = MagicMock()
-        mock_response.choices = [
-            MagicMock(
-                message=MagicMock(
-                    content=json.dumps(
-                        {
-                            "subtopics": [
-                                {"topic": "Subtopic 1", "connections": []},
-                                {"topic": "Subtopic 2", "connections": [0]},
-                            ]
-                        }
-                    )
-                )
+        # Mock the generate method on the topic_graph's llm_client instance
+        from deepfabric.schemas import GraphSubtopic, GraphSubtopics  # noqa: PLC0415
+
+        topic_graph.llm_client.generate = MagicMock(
+            return_value=GraphSubtopics(
+                subtopics=[
+                    GraphSubtopic(topic="Subtopic 1", connections=[]),
+                    GraphSubtopic(topic="Subtopic 2", connections=[0]),
+                ]
             )
-        ]
-        mock_completion.return_value = mock_response
+        )
 
         # Generate subtopics
         topic_graph.get_subtopics_and_connections(topic_graph.root, 2)
@@ -330,46 +328,28 @@ class TestGraph:
         subtopic2 = next(node for node in topic_graph.nodes.values() if node.topic == "Subtopic 2")
         assert topic_graph.root in subtopic2.parents  # Connection to root
 
-    @patch("deepfabric.graph.litellm.completion")
-    def test_get_subtopics_and_connections_retry(self, mock_completion, topic_graph, capsys):
+    def test_get_subtopics_and_connections_retry(self, topic_graph):
         """Test subtopic generation with retries."""
-        # First call fails, second succeeds
-        mock_completion.side_effect = [
-            Exception("API Error"),
-            MagicMock(
-                choices=[
-                    MagicMock(
-                        message=MagicMock(
-                            content=json.dumps(
-                                {"subtopics": [{"topic": "Subtopic", "connections": []}]}
-                            )
-                        )
-                    )
-                ]
-            ),
-        ]
+        # Mock success - LLMClient handles retries internally, so Graph only sees the final result
+        from deepfabric.schemas import GraphSubtopic, GraphSubtopics  # noqa: PLC0415
+
+        topic_graph.llm_client.generate = MagicMock(
+            return_value=GraphSubtopics(subtopics=[GraphSubtopic(topic="Subtopic", connections=[])])
+        )
 
         topic_graph.get_subtopics_and_connections(topic_graph.root, 1)
 
-        # Verify retry occurred
-        captured = capsys.readouterr()
-        assert "Error generating subtopics" in captured.out
-
-        # Verify node was added after retry
+        # Verify node was added
         assert len(topic_graph.nodes) == 2  # noqa: PLR2004
 
-    @patch("deepfabric.graph.litellm.completion")
-    @patch("deepfabric.graph.time.sleep")
     def test_get_subtopics_and_connections_max_retries(
         self,
-        mock_sleep,  # noqa: ARG002
-        mock_completion,
         topic_graph,
         capsys,  # noqa: ARG002
     ):  # noqa: ARG002
         """Test subtopic generation hitting max retries."""
         # All calls fail
-        mock_completion.side_effect = Exception("API Error")
+        topic_graph.llm_client.generate = MagicMock(side_effect=Exception("API Error"))
 
         topic_graph.get_subtopics_and_connections(topic_graph.root, 1)
 
@@ -378,31 +358,21 @@ class TestGraph:
         assert topic_graph.failed_generations[0]["node_id"] == 0
         assert "API Error" in topic_graph.failed_generations[0]["last_error"]
 
-        captured = capsys.readouterr()
-        assert "Failed to generate valid subtopics" in captured.out
-
-    @patch("deepfabric.graph.litellm.completion")
-    def test_build(self, mock_completion, topic_graph):
+    def test_build(self, topic_graph):
         """Test building the entire graph."""
         # Mock responses for each call
-        mock_completion.return_value = MagicMock(
-            choices=[
-                MagicMock(
-                    message=MagicMock(
-                        content=json.dumps(
-                            {
-                                "subtopics": [
-                                    {"topic": f"Topic {i}", "connections": []}
-                                    for i in range(topic_graph.args.graph_degree)
-                                ]
-                            }
-                        )
-                    )
-                )
-            ]
+        from deepfabric.schemas import GraphSubtopic, GraphSubtopics  # noqa: PLC0415
+
+        topic_graph.llm_client.generate = MagicMock(
+            return_value=GraphSubtopics(
+                subtopics=[
+                    GraphSubtopic(topic=f"Topic {i}", connections=[])
+                    for i in range(topic_graph.degree)
+                ]
+            )
         )
 
-        topic_graph.build()
+        list(topic_graph.build())  # Convert generator to list to complete build
 
         # With depth=2 and degree=3, we should have:
         # 1 root + 3 children + (3 * 3) grandchildren = 13 nodes
@@ -441,20 +411,21 @@ class TestGraph:
 class TestIntegration:
     """Integration tests for the Graph system."""
 
-    @patch("deepfabric.graph.litellm.completion")
-    def test_complex_graph_creation(self, mock_completion):
+    def test_complex_graph_creation(self):
         """Test creating a complex graph with cross-connections."""
-        args = GraphArguments(
-            root_prompt="Machine Learning",
-            model_name="test-model",
-            temperature=0.7,
-            graph_degree=2,
-            graph_depth=2,
-        )
-        graph = Graph(args)
+        graph_params = {
+            "topic_prompt": "Machine Learning",
+            "provider": "openai",
+            "model_name": "test-model",
+            "temperature": 0.7,
+            "degree": 2,
+            "depth": 2,
+        }
+        with patch("deepfabric.graph.LLMClient"):
+            graph = Graph(**graph_params)
 
         # First level response
-        first_response = {
+        _first_response = {
             "subtopics": [
                 {"topic": "Supervised Learning", "connections": []},
                 {"topic": "Unsupervised Learning", "connections": []},
@@ -462,14 +433,14 @@ class TestIntegration:
         }
 
         # Second level responses with cross-connections
-        supervised_response = {
+        _supervised_response = {
             "subtopics": [
                 {"topic": "Classification", "connections": []},
                 {"topic": "Regression", "connections": [2]},  # Connect to Unsupervised
             ]
         }
 
-        unsupervised_response = {
+        _unsupervised_response = {
             "subtopics": [
                 {"topic": "Clustering", "connections": [1]},  # Connect to Supervised
                 {"topic": "Dimensionality Reduction", "connections": []},
@@ -477,17 +448,34 @@ class TestIntegration:
         }
 
         # Set up mock to return different responses
-        mock_completion.side_effect = [
-            MagicMock(choices=[MagicMock(message=MagicMock(content=json.dumps(first_response)))]),
-            MagicMock(
-                choices=[MagicMock(message=MagicMock(content=json.dumps(supervised_response)))]
-            ),
-            MagicMock(
-                choices=[MagicMock(message=MagicMock(content=json.dumps(unsupervised_response)))]
-            ),
-        ]
+        from deepfabric.schemas import GraphSubtopic, GraphSubtopics  # noqa: PLC0415
 
-        graph.build()
+        graph.llm_client.generate = MagicMock(
+            side_effect=[
+                GraphSubtopics(
+                    subtopics=[
+                        GraphSubtopic(topic="Supervised Learning", connections=[]),
+                        GraphSubtopic(topic="Unsupervised Learning", connections=[]),
+                    ]
+                ),
+                GraphSubtopics(
+                    subtopics=[
+                        GraphSubtopic(topic="Classification", connections=[]),
+                        GraphSubtopic(
+                            topic="Regression", connections=[2]
+                        ),  # Connect to Unsupervised
+                    ]
+                ),
+                GraphSubtopics(
+                    subtopics=[
+                        GraphSubtopic(topic="Clustering", connections=[1]),  # Connect to Supervised
+                        GraphSubtopic(topic="Dimensionality Reduction", connections=[]),
+                    ]
+                ),
+            ]
+        )
+
+        list(graph.build())  # Convert generator to list to complete build
 
         # Verify structure
         assert len(graph.nodes) == 7  # 1 root + 2 level1 + 4 level2  # noqa: PLR2004
@@ -507,14 +495,14 @@ class TestIntegration:
 
     def test_graph_persistence_roundtrip(self):
         """Test complete save/load roundtrip with complex graph."""
-        args = GraphArguments(
-            root_prompt="Science",
-            model_name="test-model",
-            temperature=0.5,
-            graph_degree=2,
-            graph_depth=2,
-        )
-        graph = Graph(args)
+        graph_params = {
+            "topic_prompt": "Science",
+            "model_name": "test-model",
+            "temperature": 0.5,
+            "degree": 2,
+            "depth": 2,
+        }
+        graph = Graph(**graph_params)
 
         # Build a complex graph manually
         physics = graph.add_node("Physics")
@@ -536,7 +524,7 @@ class TestIntegration:
         try:
             # Save and load
             graph.save(temp_path)
-            loaded = Graph.from_json(temp_path, args)
+            loaded = Graph.from_json(temp_path, graph_params)
 
             # Verify complete structure
             assert len(loaded.nodes) == len(graph.nodes)
